# This file is autogenerated. Do not edit.
import typing
import enum

class AnnotationLexer:
    def __eq__(self, rhs: AnnotationLexer) -> bool: ...

    def __ge__(self, rhs: AnnotationLexer) -> bool: ...

    def __gt__(self, rhs: AnnotationLexer) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: AnnotationLexer, arg0: str): ...

    def __le__(self, rhs: AnnotationLexer) -> bool: ...

    def __lt__(self, rhs: AnnotationLexer) -> bool: ...

    def __ne__(self, rhs: AnnotationLexer) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    def get_error_message(self: AnnotationLexer) -> str: ...

    def has_error(self: AnnotationLexer) -> bool: ...

    def next(self: AnnotationLexer) -> Token: ...



class ClassConstructMode(enum.Enum):
    """
    Members:
      Auto : If the value is a list, pass values as `*args`. If the value is a dict,
    pass values as `** kwargs`. Otherwise pass a single value as the first argument
    to the class constructor.
      Value : Pass any value with the class's annotation as one argument to the
    class constructor
      ListAsArgs : Require that the class value is a list, and pass through all list
    values as `*args` to the class constructor
      DictAsKeywordArgs : Require that the class value is a dict, and pass through
    all dict key/value pairs as `**kwargs` to the class constructor
    """
    Auto = 0
    Value = 1
    ListAsArgs = 2
    DictAsKeywordArgs = 3


class Element:
    def __eq__(self: Element, arg0) -> bool: ...

    def __ge__(self, rhs: Element) -> bool: ...

    def __gt__(self, rhs: Element) -> bool: ...

    def __init__(self: Element, element_type: ElementType, token: Token, annotation: TokenSpan): ...

    def __le__(self, rhs: Element) -> bool: ...

    def __lt__(self, rhs: Element) -> bool: ...

    def __ne__(self: Element, arg0) -> bool: ...

    def __repr__(self: Element) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self: Element) -> str: ...

    @property
    def annotation(self) -> TokenSpan: ...

    @property
    def token(self) -> Token: ...

    @token.setter
    def token(self, value: Token): ...

    @property
    def type(self) -> ElementType: ...

    @type.setter
    def type(self, value: ElementType): ...



class ElementType(enum.Enum):
    """
    Members:
      Invalid
      Number
      Bool
      Null
      Bytes
      String
      DateTime
      ExpressionToken
      Comment
      BeginArray
      EndArray
      BeginExpression
      EndExpression
      BeginObject
      ObjectKey
      EndObject
    """
    Invalid = 0
    Number = 1
    Bool = 2
    Null = 3
    Bytes = 4
    String = 5
    DateTime = 6
    ExpressionToken = 7
    Comment = 8
    BeginArray = 9
    EndArray = 10
    BeginExpression = 11
    EndExpression = 12
    BeginObject = 13
    ObjectKey = 14
    EndObject = 15


class Encoder:
    def __eq__(self, rhs: Encoder) -> bool: ...

    def __ge__(self, rhs: Encoder) -> bool: ...

    def __gt__(self, rhs: Encoder) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: Encoder, settings: SerializerSettings, *, encode_inline: bool = False, sort_keys: bool = False, skip_keys: bool = False, decode_unicode: bool = True): ...

    def __le__(self, rhs: Encoder) -> bool: ...

    def __lt__(self, rhs: Encoder) -> bool: ...

    def __ne__(self, rhs: Encoder) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    @property
    def doc(self) -> Serializer: ...

    def encode_dict(self: Encoder, arg0: dict): ...

    def encode_sequence(self: Encoder, arg0: typing.Sequence): ...

    def encode_value(self: Encoder, arg0): ...

    def get_result(self: Encoder) -> str: ...

    def get_result_bytes(self: Encoder) -> bytes: ...

    def set_find_encoder_callback(self: Encoder, arg0): ...

    def set_find_fallback_encoder_callback(self: Encoder, arg0): ...



class ErrorInfo:
    def __bool__(self: ErrorInfo) -> bool: ...

    def __eq__(self, rhs: ErrorInfo) -> bool: ...

    def __ge__(self, rhs: ErrorInfo) -> bool: ...

    def __gt__(self, rhs: ErrorInfo) -> bool: ...

    def __hash__(self) -> int: ...

    @typing.overload
    def __init__(self: ErrorInfo): ...

    @typing.overload
    def __init__(self: ErrorInfo, message: str, start_idx: int = invalid_idx, end_idx: int = invalid_idx): ...

    def __le__(self, rhs: ErrorInfo) -> bool: ...

    def __lt__(self, rhs: ErrorInfo) -> bool: ...

    def __ne__(self, rhs: ErrorInfo) -> bool: ...

    def __repr__(self: ErrorInfo) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self: ErrorInfo) -> str: ...

    @property
    def buffer_end_idx(self) -> int: ...

    @buffer_end_idx.setter
    def buffer_end_idx(self, value: int): ...

    @property
    def buffer_start_idx(self) -> int: ...

    @buffer_start_idx.setter
    def buffer_start_idx(self, value: int): ...

    @property
    def col(self) -> int: ...

    @col.setter
    def col(self, value: int): ...

    def get_line_and_col_from_buffer(self: ErrorInfo, arg0: str) -> bool: ...

    @property
    def is_err(self) -> bool: ...

    @is_err.setter
    def is_err(self, value: bool): ...

    @property
    def line(self) -> int: ...

    @line.setter
    def line(self, value: int): ...

    @property
    def message(self) -> str: ...

    @message.setter
    def message(self, value: str): ...

    def reset(self: ErrorInfo): ...

    def to_string(self: ErrorInfo, buffer: str = '') -> str: ...



class ExpressionParseMode(enum.Enum):
    """
    Members:
      ValueList : Returns the expression as a list of values (excluding comments)
      ValueListWithComments : Returns the expression as a list of values (including
    comments)
      TokenList : Returns the expression as a list of Token objects
      SourceString : Returns the expression as a string, with the original
    formatting intact
    """
    ValueList = 0
    ValueListWithComments = 1
    TokenList = 2
    SourceString = 3


class ExpressionProxy:
    def __eq__(self, rhs: ExpressionProxy) -> bool: ...

    def __ge__(self, rhs: ExpressionProxy) -> bool: ...

    def __gt__(self, rhs: ExpressionProxy) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self, *args, **kwargs) -> typing.Any:
        """
        Initialize self.  See help(type(self)) for accurate signature.
        """

    def __le__(self, rhs: ExpressionProxy) -> bool: ...

    def __lt__(self, rhs: ExpressionProxy) -> bool: ...

    def __ne__(self, rhs: ExpressionProxy) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    def comma(self: ExpressionProxy) -> ExpressionProxy: ...

    def expression_end(self: ExpressionProxy) -> Serializer: ...

    def identifier(self: ExpressionProxy, arg0: str) -> ExpressionProxy: ...

    def identifier_or_string(self: ExpressionProxy, value: str, quote: StringQuoteMode = StringQuoteMode.Auto, decode_unicode: bool = True) -> ExpressionProxy: ...

    def op(self: ExpressionProxy, arg0: str) -> ExpressionProxy: ...

    def paren_close(self: ExpressionProxy) -> ExpressionProxy: ...

    def paren_open(self: ExpressionProxy) -> ExpressionProxy: ...

    def token(self: ExpressionProxy, arg0: TokenType) -> ExpressionProxy: ...

    def value_bool(self: ExpressionProxy, arg0: bool) -> ExpressionProxy: ...

    def value_bytes(self: ExpressionProxy, value: bytes, quote: StringQuoteMode = StringQuoteMode.Auto) -> ExpressionProxy: ...

    def value_bytes_base64(self: ExpressionProxy, value: bytes, quote: StringQuoteMode = StringQuoteMode.Auto) -> ExpressionProxy: ...

    def value_date(self: ExpressionProxy, value: datetime.date, quote: StringQuoteMode = StringQuoteMode.Auto) -> ExpressionProxy: ...

    def value_datetime(self: ExpressionProxy, value: datetime.datetime, auto_strip_time: bool = False, quote: StringQuoteMode = StringQuoteMode.Auto) -> ExpressionProxy: ...

    def value_float(self: ExpressionProxy, value: float, suffix: str = '', precision: int = 8, fixed: bool = False) -> ExpressionProxy: ...

    def value_inf(self: ExpressionProxy, negative: bool = False) -> ExpressionProxy: ...

    def value_int(self: ExpressionProxy, value: int, suffix: str = '') -> ExpressionProxy: ...

    def value_int_bin(self: ExpressionProxy, value: int, suffix: str = '') -> ExpressionProxy: ...

    def value_int_hex(self: ExpressionProxy, value: int, suffix: str = '') -> ExpressionProxy: ...

    def value_int_oct(self: ExpressionProxy, value: int, suffix: str = '') -> ExpressionProxy: ...

    def value_nan(self: ExpressionProxy) -> ExpressionProxy: ...

    def value_null(self: ExpressionProxy) -> ExpressionProxy: ...

    def value_string(self: ExpressionProxy, value: str, quote: StringQuoteMode = StringQuoteMode.Auto, decode_unicode: bool = True) -> ExpressionProxy: ...

    def value_string_raw(self: ExpressionProxy, value: str, quote: StringQuoteMode = StringQuoteMode.Auto) -> ExpressionProxy: ...

    def write(self: ExpressionProxy, arg0: str) -> ExpressionProxy: ...



class FloatLiteralType(enum.Enum):
    """
    Members:
      Finite : Normal floating point value that is not nan, inf, or -inf
      NotANumber : Equivalent to float('nan')
      PosInfinity : Equivalent to float('inf')
      NegInfinity : Equivalent to float('-inf')
    """
    Finite = 0
    NotANumber = 1
    PosInfinity = 2
    NegInfinity = 3


class JumpParser:
    def __eq__(self, rhs: JumpParser) -> bool: ...

    def __ge__(self, rhs: JumpParser) -> bool: ...

    def __gt__(self, rhs: JumpParser) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: JumpParser, arg0: str): ...

    def __le__(self, rhs: JumpParser) -> bool: ...

    def __lt__(self, rhs: JumpParser) -> bool: ...

    def __ne__(self, rhs: JumpParser) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    def get_error(self: JumpParser) -> ErrorInfo: ...

    @staticmethod
    def get_profiler_results(sort_by_runtime: bool = True) -> str:
        """
        Requires compilation with JXC_ENABLE_JUMP_BLOCK_PROFILER=1
        """

    def has_error(self: JumpParser) -> bool: ...

    def next(self: JumpParser) -> bool: ...

    def reset(self: JumpParser, arg0: str): ...

    @staticmethod
    def reset_profiler():
        """
        Requires compilation with JXC_ENABLE_JUMP_BLOCK_PROFILER=1
        """

    def stack_depth(self: JumpParser) -> int: ...

    def value(self: JumpParser) -> Element: ...



class Lexer:
    def __eq__(self, rhs: Lexer) -> bool: ...

    def __ge__(self, rhs: Lexer) -> bool: ...

    def __gt__(self, rhs: Lexer) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: Lexer, arg0: str): ...

    def __le__(self, rhs: Lexer) -> bool: ...

    def __lt__(self, rhs: Lexer) -> bool: ...

    def __ne__(self, rhs: Lexer) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    def get_error(self: Lexer) -> ErrorInfo: ...

    def get_token_pos(self: Lexer) -> tuple: ...

    def next(self: Lexer) -> Token: ...



class LogLevel(enum.Enum):
    """
    Members:
      Info
      Warning
      Error
      Fatal
    """
    Info = 0
    Warning = 1
    Error = 2
    Fatal = 3


class OwnedElement:
    def __eq__(self: OwnedElement, arg0) -> bool: ...

    def __ge__(self, rhs: OwnedElement) -> bool: ...

    def __gt__(self, rhs: OwnedElement) -> bool: ...

    def __init__(self: OwnedElement, element_type: ElementType, token: Token, annotation: OwnedTokenSpan): ...

    def __le__(self, rhs: OwnedElement) -> bool: ...

    def __lt__(self, rhs: OwnedElement) -> bool: ...

    def __ne__(self: OwnedElement, arg0) -> bool: ...

    def __repr__(self: OwnedElement) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self: OwnedElement) -> str: ...

    @property
    def annotation(self) -> OwnedTokenSpan: ...

    @property
    def token(self) -> Token: ...

    @token.setter
    def token(self, value: Token): ...

    @property
    def type(self) -> ElementType: ...

    @type.setter
    def type(self, value: ElementType): ...



class OwnedTokenSpan:
    def __bool__(self: OwnedTokenSpan) -> bool: ...

    def __eq__(self: OwnedTokenSpan, arg0: OwnedTokenSpan) -> bool: ...

    def __ge__(self, rhs: OwnedTokenSpan) -> bool: ...

    def __getitem__(self: OwnedTokenSpan, arg0: int) -> Token: ...

    def __gt__(self, rhs: OwnedTokenSpan) -> bool: ...

    def __hash__(self: OwnedTokenSpan) -> int: ...

    def __init__(self: OwnedTokenSpan, *args, source: str = ''): ...

    def __le__(self, rhs: OwnedTokenSpan) -> bool: ...

    def __len__(self: OwnedTokenSpan) -> int: ...

    def __lt__(self, rhs: OwnedTokenSpan) -> bool: ...

    def __ne__(self: OwnedTokenSpan, arg0: OwnedTokenSpan) -> bool: ...

    def __repr__(self: OwnedTokenSpan) -> str: ...

    def __setattr__(self, name, value): ...

    def __setitem__(self: OwnedTokenSpan, arg0: int, arg1: Token): ...

    def __str__(self: OwnedTokenSpan) -> str: ...

    def append(self: OwnedTokenSpan, arg0: Token): ...

    def copy(self: OwnedTokenSpan) -> OwnedTokenSpan: ...

    def pop_back(self: OwnedTokenSpan): ...

    def reset(self: OwnedTokenSpan): ...

    def source(self: OwnedTokenSpan) -> str:
        """
        The original text from the buffer that represents all the tokens in this span.
        Primarily useful for accessing the original whitespace.
        """



class Parser:
    def __eq__(self, rhs: Parser) -> bool: ...

    def __ge__(self, rhs: Parser) -> bool: ...

    def __gt__(self, rhs: Parser) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: Parser, buf, *, default_expr_parse_mode: ExpressionParseMode = ExpressionParseMode.ValueList, ignore_unknown_annotations: bool = True, ignore_unknown_number_suffixes: bool = True): ...

    def __le__(self, rhs: Parser) -> bool: ...

    def __lt__(self, rhs: Parser) -> bool: ...

    def __ne__(self, rhs: Parser) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    @property
    def default_expr_parse_mode(self) -> ExpressionParseMode: ...

    @default_expr_parse_mode.setter
    def default_expr_parse_mode(self, value: ExpressionParseMode): ...

    def get_error(self: Parser) -> ErrorInfo: ...

    def has_error(self: Parser) -> bool: ...

    @property
    def ignore_unknown_annotations(self) -> bool: ...

    @ignore_unknown_annotations.setter
    def ignore_unknown_annotations(self, value: bool): ...

    @property
    def ignore_unknown_number_suffixes(self) -> bool: ...

    @ignore_unknown_number_suffixes.setter
    def ignore_unknown_number_suffixes(self, value: bool): ...

    def parse(self: Parser): ...

    def set_annotation_constructor(self: Parser, annotation, construct): ...

    def set_custom_dict_type(self: Parser, dict_type): ...

    def set_custom_list_type(self: Parser, list_type): ...

    def set_find_construct_from_annotation_callback(self: Parser, callback): ...

    def set_find_construct_from_number_suffix_callback(self: Parser, callback): ...

    def set_number_suffix_constructor(self: Parser, suffix: str, construct): ...



class Serializer:
    def __eq__(self, rhs: Serializer) -> bool: ...

    def __ge__(self, rhs: Serializer) -> bool: ...

    def __gt__(self, rhs: Serializer) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: Serializer, settings: SerializerSettings = SerializerSettings(pretty_print=True, target_line_length=80, indent="    ", linebreak="\n", key_separator=": ", value_separator=",\n")): ...

    def __le__(self, rhs: Serializer) -> bool: ...

    def __lt__(self, rhs: Serializer) -> bool: ...

    def __ne__(self, rhs: Serializer) -> bool: ...

    def __repr__(self) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self: Serializer) -> str: ...

    def annotation(self: Serializer, arg0: str) -> Serializer: ...

    def array_begin(self: Serializer, separator: str = '') -> Serializer: ...

    def array_empty(self: Serializer) -> Serializer: ...

    def array_end(self: Serializer) -> Serializer: ...

    def clear(self: Serializer): ...

    def comment(self: Serializer, arg0: str) -> Serializer: ...

    def done(self: Serializer): ...

    def expression_begin(self: Serializer) -> ExpressionProxy: ...

    def expression_empty(self: Serializer) -> Serializer: ...

    def expression_end(self: Serializer) -> Serializer: ...

    def flush(self: Serializer): ...

    def get_result(self: Serializer) -> str: ...

    def get_settings(self: Serializer) -> SerializerSettings: ...

    def identifier(self: Serializer, arg0: str) -> Serializer: ...

    def identifier_or_string(self: Serializer, value: str, quote: StringQuoteMode = StringQuoteMode.Auto, decode_unicode: bool = True) -> Serializer: ...

    def object_begin(self: Serializer, separator: str = '') -> Serializer: ...

    def object_empty(self: Serializer) -> Serializer: ...

    def object_end(self: Serializer) -> Serializer: ...

    def object_sep(self: Serializer) -> Serializer: ...

    def sep(self: Serializer) -> Serializer: ...

    def value_bool(self: Serializer, arg0: bool) -> Serializer: ...

    def value_bytes(self: Serializer, value: bytes, quote: StringQuoteMode = StringQuoteMode.Auto) -> Serializer: ...

    def value_bytes_base64(self: Serializer, value: bytes, quote: StringQuoteMode = StringQuoteMode.Auto) -> Serializer: ...

    def value_date(self: Serializer, value: datetime.date, quote: StringQuoteMode = StringQuoteMode.Auto) -> Serializer: ...

    def value_datetime(self: Serializer, value: datetime.datetime, auto_strip_time: bool = False, quote: StringQuoteMode = StringQuoteMode.Auto) -> Serializer: ...

    def value_float(self: Serializer, value: float, suffix: str = '', precision: int = 16, fixed: bool = False) -> Serializer: ...

    def value_inf(self: Serializer, negative: bool = False) -> Serializer: ...

    def value_int(self: Serializer, value: int, suffix: str = '') -> Serializer: ...

    def value_int_bin(self: Serializer, value: int, suffix: str = '') -> Serializer: ...

    def value_int_hex(self: Serializer, value: int, suffix: str = '') -> Serializer: ...

    def value_int_oct(self: Serializer, value: int, suffix: str = '') -> Serializer: ...

    def value_nan(self: Serializer) -> Serializer: ...

    def value_null(self: Serializer) -> Serializer: ...

    def value_string(self: Serializer, value: str, quote: StringQuoteMode = StringQuoteMode.Auto, decode_unicode: bool = True) -> Serializer: ...

    def value_string_raw(self: Serializer, value: str, quote: StringQuoteMode = StringQuoteMode.Auto, tag: str = '') -> Serializer: ...

    def write(self: Serializer, arg0: str) -> Serializer: ...



class SerializerSettings:
    def __eq__(self, rhs: SerializerSettings) -> bool: ...

    def __ge__(self, rhs: SerializerSettings) -> bool: ...

    def __gt__(self, rhs: SerializerSettings) -> bool: ...

    def __hash__(self) -> int: ...

    def __init__(self: SerializerSettings, *, pretty_print: bool = True, target_line_length: int = 80, indent: str = '    ', linebreak: str = '\n', key_separator: str = ': ', value_separator: str = ',\n', default_quote: StringQuoteMode = StringQuoteMode.Double, default_float_precision: int = 12, float_fixed_precision: bool = False): ...

    def __le__(self, rhs: SerializerSettings) -> bool: ...

    def __lt__(self, rhs: SerializerSettings) -> bool: ...

    def __ne__(self, rhs: SerializerSettings) -> bool: ...

    def __repr__(self: SerializerSettings) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self) -> str: ...

    @property
    def default_float_precision(self) -> int: ...

    @default_float_precision.setter
    def default_float_precision(self, value: int): ...

    @property
    def default_quote(self) -> StringQuoteMode: ...

    @default_quote.setter
    def default_quote(self, value: StringQuoteMode): ...

    @property
    def float_fixed_precision(self) -> bool: ...

    @float_fixed_precision.setter
    def float_fixed_precision(self, value: bool): ...

    def get_target_line_length(self: SerializerSettings) -> int: ...

    @property
    def indent(self) -> str: ...

    @indent.setter
    def indent(self, value: str): ...

    @property
    def key_separator(self) -> str: ...

    @key_separator.setter
    def key_separator(self, value: str): ...

    @property
    def linebreak(self) -> str: ...

    @linebreak.setter
    def linebreak(self, value: str): ...

    @staticmethod
    def make_compact() -> SerializerSettings: ...

    @property
    def pretty_print(self) -> bool: ...

    @pretty_print.setter
    def pretty_print(self, value: bool): ...

    @property
    def target_line_length(self) -> int: ...

    @target_line_length.setter
    def target_line_length(self, value: int): ...

    @property
    def value_separator(self) -> str: ...

    @value_separator.setter
    def value_separator(self, value: str): ...



class StringQuoteMode(enum.Enum):
    """
    Members:
      Auto
      Double
      Single
    """
    Auto = 0
    Double = 1
    Single = 2


class Token:
    def __eq__(self: Token, arg0: Token) -> bool: ...

    def __ge__(self, rhs: Token) -> bool: ...

    def __gt__(self, rhs: Token) -> bool: ...

    def __init__(self: Token, type: TokenType, value = None, start_idx: int = invalid_idx, end_idx: int = invalid_idx, tag = None): ...

    def __le__(self, rhs: Token) -> bool: ...

    def __lt__(self, rhs: Token) -> bool: ...

    # [UNKNOWN TYPE tuple] '__match_args__' ('type', 'value')
    """
    Built-in immutable sequence.
    If no argument is given, the constructor returns an empty tuple.
    If iterable is specified the tuple is initialized from iterable's items.
    If the argument is a tuple, the return value is the same object.
    """

    def __ne__(self: Token, arg0: Token) -> bool: ...

    def __repr__(self: Token) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self: Token) -> str: ...

    def copy(self: Token) -> Token: ...

    @property
    def end_idx(self) -> int: ...

    @end_idx.setter
    def end_idx(self, value: int): ...

    def get_line_and_col(self: Token, arg0: str) -> tuple: ...

    @property
    def start_idx(self) -> int: ...

    @start_idx.setter
    def start_idx(self, value: int): ...

    @property
    def tag(self) -> str: ...

    @tag.setter
    def tag(self, value: str): ...

    @property
    def type(self) -> TokenType: ...

    @type.setter
    def type(self, value: TokenType): ...

    @property
    def value(self) -> str: ...

    @value.setter
    def value(self, value: str): ...



class TokenSpan:
    def __bool__(self: TokenSpan) -> bool: ...

    def __eq__(self: TokenSpan, arg0: TokenSpan) -> bool: ...

    def __ge__(self, rhs: TokenSpan) -> bool: ...

    def __getitem__(self: TokenSpan, arg0: int) -> Token: ...

    def __gt__(self, rhs: TokenSpan) -> bool: ...

    def __hash__(self: TokenSpan) -> int: ...

    def __init__(self, *args, **kwargs) -> typing.Any:
        """
        Initialize self.  See help(type(self)) for accurate signature.
        """

    def __le__(self, rhs: TokenSpan) -> bool: ...

    def __len__(self: TokenSpan) -> int: ...

    def __lt__(self, rhs: TokenSpan) -> bool: ...

    def __ne__(self: TokenSpan, arg0: TokenSpan) -> bool: ...

    def __repr__(self: TokenSpan) -> str: ...

    def __setattr__(self, name, value): ...

    def __str__(self: TokenSpan) -> str: ...

    def copy(self: TokenSpan) -> OwnedTokenSpan: ...

    def source(self: TokenSpan) -> str:
        """
        The original text from the buffer that represents all the tokens in this span.
        Primarily useful for accessing the original whitespace.
        """



class TokenType(enum.Enum):
    """
    Members:
      Invalid
      Comment
      Identifier
      True_
      False_
      Null
      Number
      String
      ByteString
      DateTime
      Colon
      Equals
      Comma
      Period
      BraceOpen
      BraceClose
      SquareBracketOpen
      SquareBracketClose
      AngleBracketOpen
      AngleBracketClose
      ParenOpen
      ParenClose
      ExclamationPoint
      Asterisk
      QuestionMark
      AtSymbol
      Pipe
      Ampersand
      Percent
      Semicolon
      Plus
      Minus
      Slash
      Backslash
      Caret
      Tilde
      Backtick
      LineBreak
      EndOfStream
    """
    Invalid = 0
    Comment = 1
    Identifier = 2
    True_ = 3
    False_ = 4
    Null = 5
    Number = 6
    String = 7
    ByteString = 8
    DateTime = 9
    Colon = 10
    Equals = 11
    Comma = 12
    Period = 13
    BraceOpen = 14
    BraceClose = 15
    SquareBracketOpen = 16
    SquareBracketClose = 17
    AngleBracketOpen = 18
    AngleBracketClose = 19
    ParenOpen = 20
    ParenClose = 21
    ExclamationPoint = 22
    Asterisk = 23
    QuestionMark = 24
    AtSymbol = 25
    Pipe = 26
    Ampersand = 27
    Percent = 28
    Semicolon = 29
    Plus = 30
    Minus = 31
    Slash = 32
    Backslash = 33
    Caret = 34
    Tilde = 35
    Backtick = 36
    LineBreak = 37
    EndOfStream = 38


def _jxc_assert(arg0: bool, arg1: str): ...

def date_to_iso8601(dt: datetime.date) -> str: ...

def datetime_to_iso8601(dt: datetime.datetime, auto_strip_time: bool = False) -> str: ...

def datetime_token_is_date(token: Token) -> bool:
    """
    Checks if the given datetime token is just a date (no time data)
    """

def datetime_token_is_datetime(token: Token) -> bool:
    """
    Checks if the given datetime token includes both date and time data
    """

def debug_bytes_repr(value: bytes, quote_char: str = '"') -> str:
    """
    helper for creating a debug representation for bytes
    """

def debug_char_repr(codepoint, quote_char: str = '`') -> str:
    """
    helper for creating a debug representation for a single codepoint
    """

def debug_string_repr(value: str, quote_char: str = '"') -> str:
    """
    helper for creating a debug representation for a string
    """

def element_can_contain_annotation(arg0: ElementType) -> bool:
    """
    Returns true if a given element type is allowed to have an annotation
    """

def element_can_contain_value(arg0: ElementType) -> bool:
    """
    Returns true if a given element type can contain one or more value tokens. (eg.
    a Number element always contains a value token, but a Null element does not
    require one)
    """

def element_is_expression_value_type(arg0: ElementType) -> bool:
    """
    Returns true if a given element is valid for use inside an expression container
    """

def element_is_value_type(arg0: ElementType) -> bool:
    """
    Returns true if a given element is a valid value type (includes container-begin
    elements)
    """

invalid_idx: int = 18446744073709551615
def is_ascii_escape_char(codepoint, quote_char: str = '\x00') -> bool:
    """
    checks if a character should be escaped as ascii (eg. ' ')
    """

def is_debug_build() -> bool:
    """
    Returns True if pyjxc was compiled with JXC_DEBUG=1
    """

def is_profiler_enabled() -> bool:
    """
    Returns True if pyjxc was compiled with JXC_ENABLE_JUMP_BLOCK_PROFILER=1
    """

def is_renderable_ascii_char(codepoint) -> bool:
    """
    Checks if a given codepoint is a renderable ASCII char (eg. not a control
    character). NB: Returns true for space.
    """

def is_valid_identifier(arg0: str) -> bool: ...

def is_valid_identifier_char(arg0: str) -> bool: ...

def is_valid_identifier_first_char(arg0: str) -> bool: ...

def is_valid_object_key(arg0: str) -> bool: ...

def parse_bytes_token(arg0: Token): ...

def parse_date_token(token: Token):
    """
    Parses a date token into a Date value. This will return an error if the token
    includes time data.
    """

def parse_datetime_token(token: Token, require_time_data: bool = False):
    """
    Parses a date token into a DateTime value.
    If require_time_data is true, this will return an error if the token does not
    include time info.
    If require_time_data is false and the token does not include time data,
    out_datetime will have a time of 00:00:00Z.
    """

def parse_number_token(number_token: Token) -> tuple:
    """
    Parses a number token to a float or int value. Always returns a 2-tuple in the
    form (value_or_errorinfo, number_suffix)
    """

@typing.overload
def parse_string_token(arg0: Token): ...

@typing.overload
def parse_string_token(arg0: str) -> str: ...

def set_custom_assert_handler(arg0): ...

def set_custom_log_handler(arg0): ...

def split_number_token_value(arg0: Token):
    """
            Splits a number token into its component parts, and returns a tuple in
    the form (sign, prefix, number, exponent, suffix, float_type).
            All return types are strings except exponent, which is an int.
    """

def token_type_from_symbol(symbol: str) -> TokenType: ...

def token_type_has_value(arg0: TokenType) -> bool: ...

def token_type_to_symbol(arg0: TokenType) -> str: ...




from _pyjxc import (
    # constants
    __version__,
    invalid_idx,

    # enums
    TokenType,
    ElementType,
    ExpressionParseMode,
    ClassConstructMode,
    StringQuoteMode,
    FloatLiteralType,

    # classes
    ErrorInfo,
    Parser,
    Token,
    Element,
    Serializer,
    SerializerSettings,
    Encoder,

    # functions
    token_type_has_value,
    token_type_to_symbol,
    split_number_token_value,
    element_can_contain_annotation,
    element_can_contain_value,
    element_is_expression_value_type,
    element_is_value_type,
)

from . import (encode, decode)
from .decode import (ParseError, load, loads)
from .encode import (EncodeFunc, dump, dumps)
from .util import (get_class_path, find_class)



import enum
import typing
import dataclasses

import _pyjxc
from _pyjxc import Token, TokenType, Element, ElementType, ExpressionParseMode, Parser, ErrorInfo, ClassConstructMode

from .util import find_class


class ParseError(ValueError):
    def __init__(self, msg: str, err_info: typing.Optional[ErrorInfo] = None):
        self.err_info = err_info
        super().__init__(msg)


    @classmethod
    def make(cls, msg: str, obj: typing.Union[None, Element, Token], buffer: str):
        if isinstance(obj, Token):
            start_idx, end_idx = obj.start_idx, obj.end_idx
        elif isinstance(obj, Element):
            start_idx, end_idx = obj.token.start_idx, obj.token.end_idx
        else:
            start_idx, end_idx = _pyjxc.invalid_idx, _pyjxc.invalid_idx
        err = ErrorInfo(msg, start_idx, end_idx)
        err.get_line_and_col_from_buffer(buffer)
        return cls(err.to_string(buffer), err_info=err)



def lex(value: str) -> typing.Iterable[Token]:
    """
    JXC lexer - iterates over raw JXC tokens.
    Useful for writing a fully custom parser, or even tokenizing syntax that is similar to JXC syntax.
    """
    lexer = _pyjxc.Lexer(value)
    tok: Token = lexer.next()
    while tok.type != TokenType.EndOfStream:
        if tok.type == TokenType.Invalid:
            err: ErrorInfo = lexer.get_error()
            err.get_line_and_col_from_buffer(value)
            raise ParseError(err.to_string(value), err_info=err)
        yield tok
        tok = lexer.next()


def lex_annotation(value: str) -> typing.Iterable[Token]:
    """
    JXC lexer, specifically for lexing a string containing an annotation.
    """
    lexer = _pyjxc.AnnotationLexer(value)
    tok: Token = lexer.next()
    while tok.type != TokenType.EndOfStream:
        if tok.type == TokenType.Invalid:
            raise ParseError(f'Annotation parse error: {lexer.get_error_message()}')
        yield tok
        tok = lexer.next()


def elements(value: str) -> typing.Iterable[Element]:
    """
    JXC parser - iterates over JXC elements.
    This is the core building block in writing a customized parser for custom data types.
    """
    parser = _pyjxc.JumpParser(value)
    while parser.next():
        ele: Element = parser.value()
        if ele.type == ElementType.Invalid:
            break
        else:
            yield ele

    if parser.has_error():
        err = parser.get_error()
        err.get_line_and_col_from_buffer(value)
        raise ParseError(err.to_string(value), err)


TokenLike = typing.Union[None, str, TokenType, tuple[str, TokenType]]
ValueConstructorCallable = typing.Union[type, typing.Callable[[typing.Any], typing.Any]]
ValueConstructor = typing.Union[
    None,
    ValueConstructorCallable,
    tuple[ValueConstructorCallable, ClassConstructMode],
    tuple[ValueConstructorCallable, ExpressionParseMode],
    tuple[ValueConstructorCallable, ClassConstructMode, ExpressionParseMode]
]


def loads(buf: typing.Union[str, bytes], *,
        decode_inline=False,
        decode_enum=False,
        decode_dataclass=False,
        default_expr_parse_mode=ExpressionParseMode.ValueList,
        ignore_unknown_annotations=True,
        ignore_unknown_number_suffixes=True,
        annotation_hooks: typing.Optional[typing.Iterable[tuple[typing.Union[TokenLike, typing.Iterable[TokenLike]], ValueConstructor]]] = None,
        number_suffix_hooks: typing.Optional[typing.Iterable[tuple[str, ValueConstructor]]] = None):
    parser = Parser(buf,
        default_expr_parse_mode=default_expr_parse_mode,
        ignore_unknown_annotations=ignore_unknown_annotations,
        ignore_unknown_number_suffixes=ignore_unknown_number_suffixes)

    if annotation_hooks is not None:
        for anno, construct_func in annotation_hooks:
            if isinstance(anno, str) and not _pyjxc.is_valid_identifier(anno):
                # If the annotation is a string that isn't a plain identifier, it's not going to work, because
                # _pyjxc.Parser.set_annotation_constructor treats strings as single token.
                # Split the string into tokens first.
                # This has the (intended) side effect of ignoring whitespace in annotation tokens,
                # so the annotations "list<f32, 8>" and "list< f32 , 8 >" are equal.
                anno = list(lex_annotation(anno))
            parser.set_annotation_constructor(anno, construct_func)

    if number_suffix_hooks is not None:
        for suffix, construct_func in number_suffix_hooks:
            parser.set_number_suffix_constructor(suffix, construct_func)

    if decode_inline or decode_enum or decode_dataclass:
        def lookup_class(ele: Element) -> ValueConstructor:
            if anno := ele.annotation.source():
                if cls := find_class(anno):
                    if decode_enum and issubclass(cls, enum.Enum):
                        return (cls, ClassConstructMode.Value)
                    elif decode_inline and hasattr(cls, '_jxc_decode'):
                        return cls
                    elif decode_dataclass and dataclasses.is_dataclass(cls):
                        return (cls, ClassConstructMode.DictAsKeywordArgs)
            return None
        parser.set_find_construct_from_annotation_callback(lookup_class)

    return parser.parse()


def load(fp: typing.TextIO, **kwargs) -> typing.Any:
    return loads(fp.read(), **kwargs)



#!/usr/bin/env python3
import os
import sys


def find_defines(filename):
    """
    Generator that returns all #define lines in a file.
    NB. This does not return valid output if you care about function-style macros, preprocessor conditionals, or multi-line macros.
    """
    preproc_token = '#'
    def_token = 'define'
    with open(filename) as fp:
        for line in fp:
            line = line.strip()
            if not line.startswith(preproc_token):
                continue
            line = line[len(preproc_token):].strip()
            if not line.startswith(def_token):
                continue
            line = line[len(def_token):].strip()
            name_end_idx = line.find(' ')
            yield (line[:name_end_idx], line[name_end_idx:].strip()) if (name_end_idx >= 0) else (line, '')


if __name__ == "__main__":
    version = {
        'JXC_VERSION_MAJOR': None,
        'JXC_VERSION_MINOR': None,
        'JXC_VERSION_PATCH': None,
    }

    repo_root = os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..'))
    jxc_core_path = os.path.join(repo_root, 'jxc', 'include', 'jxc', 'jxc_core.h')
    if not os.path.exists(jxc_core_path):
        print(f'Error: jxc_core.h not found. Tried {jxc_core_path!r}', file=sys.stderr)
        sys.exit(1)

    for def_name, def_value in find_defines(jxc_core_path):
        if def_name in version:
            version[def_name] = def_value
        if all(v is not None for v in version.values()):
            break

    print(".".join(str(v) for v in (
        version['JXC_VERSION_MAJOR'],
        version['JXC_VERSION_MINOR'],
        version['JXC_VERSION_PATCH'],
    )))



import os
import sys
import time
import typing
import contextlib
import _pyjxc


TokenSpanType = typing.Union[_pyjxc.TokenSpan, _pyjxc.OwnedTokenSpan]


def get_main_module_name() -> str:
    """
    Helper to get the name of the __main__ module, if any
    """
    if (main_module := sys.modules.get("__main__", None)) and hasattr(main_module, '__file__'):
        return os.path.basename(main_module.__file__).removesuffix(".py")
    return '__main__'


_MAIN_MODULE_NAME = get_main_module_name()
_MAIN_MODULES = set(["__main__", get_main_module_name()])


def get_class_path(cls: type) -> str:
    """
    Returns a string representing a class that can be used to retrieve that class
    from a string using find_class.
    """
    if cls.__module__ == "__main__":
        return cls.__name__
    elif cls.__module__ == "builtins":
        return cls.__name__
    else:
        return f"{cls.__module__}.{cls.__name__}"


_FIND_CLASS_CACHE: dict[str, tuple[str, str]] = {}


def find_class(path: str) -> typing.Optional[type]:
    """
    Looks up a class using a path in the form `module.class_name`, importing the module if needed.
    If no module name is included, looks up the name in builtins or the __main__ module.
    """
    # We don't want to cache the actual classes, because then we're holding references to those classes and that can
    # be problematic for some modules.
    # Instead, we cache the module/class_name split - just the pair of strings. This seems like a micro-optimization,
    # but in practice this is a 5x speedup, which makes a difference when we're calling this thousands of times while
    # parsing a large JXC document.
    if cache_entry := _FIND_CLASS_CACHE.get(path, None):
        module_path, class_name = cache_entry
        if (mod := sys.modules.get(module_path, None)) and (cls := getattr(mod, class_name, None)):
            return cls
        return getattr(__import__(module_path, fromlist=[class_name]), class_name, None)

    # no cache entry exists - slow path
    parts = path.split(".")
    num_parts = len(parts)
    if num_parts == 0:
        return None
    elif num_parts == 1:
        class_name = parts[0]
        # no module listed, assume it's in the __main__ or builtins module
        if (main_module := sys.modules.get('__main__', None)) and (cls := getattr(main_module, class_name, None)):
            _FIND_CLASS_CACHE[path] = ('__main__', class_name)
            return cls
        else:
            _FIND_CLASS_CACHE[path] = ('builtins', class_name)
            return getattr(sys.modules['builtins'], class_name, None)
    elif num_parts == 2 and parts[0] == "builtins":
        class_name = parts[1]
        # get the class from the builtins module
        _FIND_CLASS_CACHE[path] = ('builtins', class_name)
        return getattr(sys.modules['builtins'], class_name, None)
    elif num_parts == 2 and parts[0] in _MAIN_MODULES:
        class_name = parts[1]
        # get the class from the __main__ module
        if main_module := sys.modules.get('__main__', None):
            _FIND_CLASS_CACHE[path] = ('__main__', class_name)
            return getattr(main_module, class_name, None)
        return None
    elif num_parts >= 2:
        module_path = ".".join(parts[:-1])
        class_name = parts[-1]
        _FIND_CLASS_CACHE[path] = (module_path, class_name)
        return getattr(__import__(module_path, fromlist=[class_name]), class_name, None)

    return None




import os
import sys
import typing
import subprocess
from setuptools import setup

# see if there is a system pybind11 install available
try:
    import pybind11 as _
except ImportError:
    # no system pybind11 install available, fall back on the bundled copy
    sys.path.insert(0, "./subprojects/pybind11")

from pybind11.setup_helpers import Pybind11Extension, build_ext


jxc_version = subprocess.check_output([sys.executable, './tools/version.py']).decode('utf-8').strip()
build_root_dir = os.path.normpath(os.path.dirname(os.path.abspath(__file__)))


def parse_version(ver: str) -> typing.Tuple[int, int, typing.Union[int, str]]:
    parts = list(ver.split('.'))
    if len(parts) == 1 and parts[0].isnumeric():
        return (int(parts[0]), 0, 0)
    elif len(parts) in (2, 3) and parts[0].isnumeric() and parts[1].isnumeric():
        patch_ver = 0
        if len(parts) > 2:
            patch_ver = int(parts[2]) if parts[2].isnumeric() else parts[2]
        return (int(parts[0]), int(parts[1]), patch_ver)
    raise ValueError(f'Value {ver!r} is not a version number')


def executable_exists(path):
    return os.path.exists(path) and os.access(path, os.X_OK)


def get_re2c_version(re2c_path: str) -> str:
    assert executable_exists(re2c_path)
    try:
        re2c_version = subprocess.check_output([re2c_path, '--version']).decode('utf-8').strip()
    except subprocess.CalledProcessError:
        return ''
    if re2c_version.startswith('re2c'):
        re2c_version = re2c_version[4:].strip()
    if re2c_version.endswith('(debug)'):
        re2c_version = re2c_version[:-7].strip()
    return re2c_version


def find_re2c() -> typing.Optional[str]:
    re2c_path = None
    if os.name == 'nt':
        re2c_path = os.path.join(build_root_dir, 'tools', 're2c.exe')
    else:
        # first, see if there's a re2c binary in the tools directory
        re2c_path = os.path.join(build_root_dir, 'tools', 're2c')

    # we have a bundled re2c executable
    if executable_exists(re2c_path):
        return re2c_path

    # if that doesn't exist, ask the shell if we have one available
    try:
        if os.name == 'nt':
            search_cmd = ['where', 're2c.exe']
        else:
            search_cmd = ['which', 're2c']
        re2c_path = subprocess.check_output(search_cmd).decode('utf-8').strip()
        if executable_exists(re2c_path):
            return re2c_path
    except subprocess.CalledProcessError:
        pass

    return None


def run_re2c(re2c_path: str, input_file: str):
    assert os.path.exists(input_file)
    input_file = os.path.relpath(input_file, start=build_root_dir)
    assert os.path.exists(input_file)

    args = [re2c_path, '-W', '--verbose', '--utf8', '--input-encoding', 'utf8']
    if os.name == 'nt':
        args += ['--location-format', 'msvc']
    else:
        args += ['--location-format', 'gnu']
    
    re2c_args = args + ['-o', f'{input_file}.cpp', input_file]
    print(' '.join(re2c_args))
    subprocess.check_call(re2c_args)


ext_modules = [
    Pybind11Extension(
        "_pyjxc",
        cxx_std=17,
        sources=[
            # JXC core library
            "jxc/src/jxc_core.cpp",
            "jxc/src/jxc_lexer.cpp",
            "jxc/src/jxc_lexer_gen.re.cpp",
            "jxc/src/jxc_parser.cpp",
            "jxc/src/jxc_serializer.cpp",
            "jxc/src/jxc_string.cpp",
            "jxc/src/jxc_util.cpp",

            # JXC Python bindings
            "bindings/python/src/jxc_pyparser.cpp",
            "bindings/python/src/jxc_pyserializer.cpp",
            "bindings/python/src/jxc_python.cpp",
        ],
        include_dirs=[
            "jxc/include",
            "bindings/python/src",
            "subprojects/fast_float/include",
            "subprojects/unordered_dense/include",
        ],
        define_macros = [
            ('JXC_DEBUG', 0),
            ('JXC_ENABLE_RELEASE_ASSERTS', 1),
            ('JXC_ENABLE_DEBUG_ASSERTS', 1),  # NB. These are only active when JXC_DEBUG == 1
            ('JXC_ENABLE_DEBUG_ALLOCATOR', 0),
            ('JXC_ENABLE_JUMP_BLOCK_PROFILER', 0),
        ],
    ),
]


class build_ext_jxc(build_ext):
    per_platform_compile_args = {
        '_pyjxc': {
            'unix': ['-Wno-reorder'],
        }
    }

    def build_extension(self, ext):
        generated_lexer_path = os.path.join(build_root_dir, 'jxc', 'src', 'jxc_lexer_gen.re.cpp')

        re2c_path = find_re2c()
        re2c_ver = get_re2c_version(re2c_path) if re2c_path else ''
        if re2c_path and parse_version(re2c_ver)[0] >= 3:
            run_re2c(re2c_path, os.path.join(build_root_dir, 'jxc', 'src', 'jxc_lexer_gen.re'))
        elif re2c_path:
            print(f'Warning: re2c version {re2c_ver} found, but JXC requires re2c >= 3.0. Using pre-generated lexer source file.')
        elif os.path.exists(generated_lexer_path):
            print('Warning: re2c not found, but generated lexer source already exists. Using pre-generated lexer source file.')
        else:
            raise FileNotFoundError(f'{generated_lexer_path!r} not found, and re2c could not be found to generate it.')

        assert os.path.exists(generated_lexer_path)
        if extra_args := self.per_platform_compile_args.get(ext.name, None):
            if args := extra_args.get(self.compiler.compiler_type, None):
                if not ext.extra_compile_args:
                    ext.extra_compile_args = []
                ext.extra_compile_args += args
        return super().build_extension(ext)


# verify that jxc_version is a valid version number
try:
    parse_version(jxc_version)
except ValueError as e:
    raise Exception(f'Invalid JXC version {jxc_version!r}: {e}')


setup(
    name="jxc",
    version=jxc_version,
    #long_description="",
    cmdclass={'build_ext': build_ext_jxc},
    ext_modules=ext_modules,
    packages=['_pyjxc', 'jxc'],
    package_dir={
        '_pyjxc': './bindings/python/_pyjxc',
        'jxc': './bindings/python/jxc',
    },
    package_data={
        '_pyjxc': ['py.typed', '__init__.pyi'],
        'jxc': ['py.typed'],
    },
    zip_safe=False,
    python_requires=">=3.7",
)



import os
import sys
import re
import math
import enum
import typing
import dataclasses

from _pyjxc import Serializer, SerializerSettings, StringQuoteMode, Encoder, FloatLiteralType
from .util import get_class_path


EncodeFunc = typing.Callable[[Serializer, Encoder, typing.Any], None]


_OBJECT_KEY_IDENT_REGEX = re.compile(r'([a-zA-Z_$*][a-zA-Z0-9_$*]*)([\.\-][a-zA-Z_$*][a-zA-Z0-9_$*]*)*')
_EXPR_IDENT_REGEX = re.compile(r'[a-zA-Z_$][a-zA-Z0-9_$]*')


def is_valid_identifier_for_object_key(val: str) -> bool:
    """
    Returns True if the specified string is valid for use as an unquoted object key.
    """
    return _OBJECT_KEY_IDENT_REGEX.match(val) is not None



def is_valid_expression_identifier(val: str) -> bool:
    """
    Returns True if the specified string is valid for use as an unquoted identifer inside an expression.
    """
    return _EXPR_IDENT_REGEX.match(val) is not None



def enum_encoder(doc: Serializer, enc: Encoder, val: enum.Enum):
    doc.annotation(get_class_path(val.__class__))
    if isinstance(val, enum.IntEnum):
        doc.value_int(val.value)
    elif val.value is None:
        doc.value_null()
    elif isinstance(val.value, int):
        doc.value_int(val.value)
    elif isinstance(val.value, float):
        doc.value_float(val.value)
    elif isinstance(val.value, str):
        doc.value_string(val.value)
    else:
        raise TypeError(f"Failed to serialize enum type {type(val).__name__}: "
            "value type {type(val.value).__name__} is not supported")



def dataclass_encoder(doc: Serializer, enc: Encoder, val: typing.Any):
    doc.annotation(get_class_path(val.__class__))
    doc.object_begin()
    if hasattr(val.__class__, '__annotations__') and len(val.__class__.__annotations__) > 0:
        for var_name, var_type in val.__class__.__annotations__.items():
            if is_valid_identifier_for_object_key(var_name):
                doc.identifier(var_name)
            else:
                doc.value_string(var_name)
            doc.object_sep()
            enc.encode_value(getattr(val, var_name, None))
    else:
        for var_name, var_value in val.__dict__.items():
            if is_valid_identifier_for_object_key(var_name):
                doc.identifier(var_name)
            else:
                doc.value_string(var_name)
            doc.object_sep()
            enc.encode_value(var_value)
    doc.object_end()



def dumps(obj, *,
        indent: typing.Union[None, str, int] = None,
        sort_keys: bool = False,
        skip_keys: bool = False,
        decode_unicode: bool = False,
        check_circular: bool = True,
        separators: typing.Optional[tuple[str, str]] = None,
        encode_enum: bool = False,
        encode_dataclass: bool = False,
        encode_inline: bool = True,
        encode_class: typing.Optional[dict[type, EncodeFunc]] = None,
        default: typing.Optional[EncodeFunc] = None,
        quote: StringQuoteMode = StringQuoteMode.Double,
        as_bytes: bool = False) -> typing.Union[str, bytes]:
    """
    Simple JXC serializer with an API similar to json.dumps
    """
    if not indent:
        settings = SerializerSettings.make_compact()
    else:
        settings = SerializerSettings()
        settings.pretty_print = True
        if isinstance(indent, str):
            settings.indent = indent
        elif isinstance(indent, int):
            settings.indent = ' ' * indent
        else:
            raise TypeError(f'indent must be None, int, or str. Got {type(indent)}')

    settings.default_quote = quote

    if separators is not None:
        if not isinstance(separators, tuple) or len(separators) != 2:
            raise ValueError(f'separators should be a 2-tuple or None, got {separators!r}')
        settings.value_separator = separators[0]
        settings.key_separator = separators[1]
    
    #TODO: implement check_circular

    encoder = Encoder(
        settings,
        encode_inline=encode_inline,
        sort_keys=sort_keys,
        skip_keys=skip_keys,
        decode_unicode=decode_unicode)

    if encode_class is not None or encode_enum or encode_dataclass:
        def find_encoder(value: typing.Any) -> typing.Optional[typing.Callable[[Serializer, Encoder, typing.Any], None]]:
            if encode_inline and hasattr(value, '_jxc_encode'):
                # object has an inline encoder, so skip the enum/dataclass auto-encoding and let the class handle it
                return None
            elif encode_enum and isinstance(value, enum.Enum):
                return enum_encoder
            elif encode_dataclass and dataclasses.is_dataclass(value) and not isinstance(value, type):
                return dataclass_encoder
            elif encode_class is not None and (class_encoder := encode_class.get(type(value))):
                return class_encoder
            return None
        encoder.set_find_encoder_callback(find_encoder)

    if default is not None:
        def fallback_encoder(value: typing.Any):
            return default
        encoder.set_find_fallback_encoder_callback(fallback_encoder)

    encoder.encode_value(obj)
    return encoder.get_result_bytes() if as_bytes else encoder.get_result()


def dump(obj: typing.Any, fp: typing.TextIO, **kwargs):
    fp.write(dumps(obj, **kwargs))

